{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
{----------------------------------------------------------------------------}
{ This is an implementation of Halley's method for approximating a zero of   }
{ a twice continuously differentiable function.                              }
{                                                                            }
{    given:      the function f(x) and the starting value x[0]               }
{    iteration:  x[k+1] := x[k] + a[k]/(1+b[k]/2)          \                 }
{    with:       a[k]   := - f(x[k])/f'(x[k])               ) k=0,1,2,...    }
{                b[k]   := a[k]*f''(x[k])/f'(x[k])         /                 }
{                                                                            }
{ All real operations are replaced by interval operations. All function and  }
{ derivative evaluations are calculated by differentiation arithmetic.       }
{ This approximate method can NOT guarantee to find and enclose a zero of    }
{ the function 'f' because we have enclosed only roundoff errors, not        }
{ truncation errors.                                                         }
{----------------------------------------------------------------------------}
program ddf_ex_halley_method;
use
  i_ari,     { Interval arithmetic        }
  ddf_ari;   { Differentiation arithmetic }
const
  MaxIter = 100;
var
  x, a, b, fx, dfx, ddfx : interval;
  n                      : integer;
  start                  : real;

function f (xD: DerivType) : DerivType;
begin
  f  := exp(xD) * sin(4*xD)
end;

begin
  writeln('Halley''s method for the function  f(x) = exp(x) * sin(4*x)');
  writeln;
  write('Starting value = ');  start:= 1.25;  writeln(start:10);
  writeln;  writeln('Iteration:');
  x:= start;  ddfEval(f,x,fx,dfx,ddfx);  n:= 0;
  repeat
    n:= n + 1;
    writeln('x                : ', x);
    writeln('f(x)             : ', fx);
    a:= - fx / dfx;  b:= ddfx / dfx * a;
    x:= x + a / (1 + 0.5*b);
    ddfEval(f,x,fx,dfx,ddfx);
  until (0 in fx) or (n >= MaxIter);
  writeln;
  writeln('Approximate zero : ', x);
  writeln('Function value   : ', fx);
  writeln;
  writeln('Expected zero    : 1.570796326794896619202...');
end.
