{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
{----------------------------------------------------------------------------}
{ This program uses module 'gop1' to compute the global optimizers of the    }
{ functions                                                                  }
{                                                                            }
{    f(x) = (x + sin(x)) * exp(-sqr(x)).                                     }
{                                                                            }
{ and                                                                        }
{              5                                                             }
{    g(x) = - sum ( k * sin((k+1)*x + k) )                                   }
{             k=1                                                            }
{                                                                            }
{ A starting interval and a tolerance must be entered.                       }
{----------------------------------------------------------------------------}
program gop1_ex;
use
  i_ari,    { Interval arithmetic                 }
  ddf_ari,  { Differentiation arithmetic          }
  i_util,   { Interval utilities                  }
  b_util,   { Boolean utilities                   }
  gop1;     { Global optimization one-dimensional }
const
  n = 20;  { Maximum number of optimizers to be computed }

function f (x : DerivType) : DerivType;
begin
  f := (x + sin(x)) * exp(-sqr(x));
end;

function g (x : DerivType) : DerivType;
var
 s : DerivType;
 k : integer;
begin
  s := DerivConst(0);
  for k:=1 to 5 do
    s := s + k * sin ( (k+1) * x + k );
  g := - s;
end;

{---------------------------------------------------------------------------}
{ Procedure for printing and reading information to call the procedure      }
{ 'AllGOp1'. This procedure must be called with the function 'f' and a      }
{ string 'Name' containing a textual description of that function.          }
{---------------------------------------------------------------------------}
procedure compute (function f(x:DerivType): DerivType; Name: string);
var
  SearchInterval, Minimum   : interval;
  Tolerance                 : real;
  Opti                      : ivector[1..n];
  Unique                    : bvector[1..n];
  NumberOfOptis, i, ErrCode : integer;
begin
  writeln('Computing all global minimizers of the function  ', Name);
  write('Search interval      : '); read(SearchInterval);
  write('Tolerance (relative) : '); read(Tolerance);
  writeln;
  AllGOp1(f, SearchInterval, Tolerance,
             Opti, Unique, NumberOfOptis, Minimum, ErrCode);
  for i:=1 to NumberOfOptis do
  begin
    writeln(Opti[i]);
    if unique[i] then
      writeln('encloses a locally unique candidate for a global minimizer!')
    else
      writeln('may contain a local or global minimizer!')
  end;
  writeln;
  if (NumberOfOptis <> 0) then
  begin
    writeln(Minimum);  writeln('encloses the global minimum value!');
  end;
  if ErrCode <> 0 then writeln(AllGOp1ErrMsg(ErrCode));
  writeln;  writeln(NumberOfOptis:1, ' interval enclosure(s)');  writeln;
  if (NumberOfOptis = 1) and (unique[1]) then
    writeln('We have validated that there is a unique global optimizer!');
end;

begin { Main program }
  compute(f,  '(x + SIN(x))*EXP(-x^2)');  writeln;  writeln;
  compute(g, '-SUM(k*SIN((k+1)*x+k),k,1,5)');
end.
