{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
{----------------------------------------------------------------------------}
{ This program uses module 'gop' to compute the global optimizers of the     }
{ function of Branin                                                         }
{                                                                            }
{    fB(x) = sqr(5/pi*x[1] - 51/(40*sqr(pi))*sqr(x[1]) + x[2] - 6)           }
{                                                                            }
{            + 10*(1-1/8/pi)*cos(x[1]) + 10                                  }
{                                                                            }
{ and the function of Levy                                                   }
{             5                         5                                    }
{    fL(x) = sum i cos((i-1)x[1] + i)  sum j cos((j+1)x[2] + j)              }
{            i=1                       j=1                                   }
{                                                                            }
{            + sqr(x[1] + 1.42513) + sqr(x[2] + 0.80032)                     }
{                                                                            }
{ A starting interval and a tolerance must be entered.                       }
{----------------------------------------------------------------------------}
program gop_ex;

use
  i_ari,     { Interval arithmetic               }
  mvi_ari,   { Interval matrix/vector arithmetic }
  hess_ari,  { Differentiation arithmetic        }
  i_util,    { Interval utilities                }
  b_util,    { Boolean utilities                 }
  gop;       { Global optimization               }
const
  n    = 20;   { Maximum number of optimizers to be computed }
  fDim = 2;    { Dimension of sample functions               }

function fBranin (x: HTvector) : HessType[0..fDim,0..fDim];
begin
  fBranin := sqr(5/Pi*x[1] - 51/(40*sqr(Pi))*sqr(x[1]) + x[2] - 6)
             + 10*(1-1/8/Pi)*cos(x[1]) + 10;
end;

function fLevy (x: HTvector) : HessType[0..fDim,0..fDim];
var
  isum, jsum : HessType[0..fDim,0..fDim];
  i          : integer;
begin
  isum := 0; jsum := 0;
  for i:=1 to 5 do
  begin
    isum := isum + i*cos((i-1)*x[1] + i);
    jsum := jsum + i*cos((i+1)*x[2] + i);
  end;
  fLevy := isum * jsum + sqr(x[1] + intval(142513)/100000) { Avoid real con- }
                       + sqr(x[2] + intval(80032)/100000); { version error   }
end;

{----------------------------------------------------------------------------}
{ Procedure for printing and reading informations to call the procedure      }
{ 'AllZeros'. This procedure must be called with the function 'f', a string  }
{ 'Name' containing textual description of that function, and an integer     }
{ 'dim' specifying the dimension of the problem.                             }
{----------------------------------------------------------------------------}
procedure compute (function f(x:HTvector): HessType[0..ub(x),0..ub(x)];
                                     Name: string;
                                      dim: integer);
var
  SearchInterval            : ivector[1..dim];
  Minimum                   : interval;
  Tolerance                 : real;
  Opti                      : imatrix[1..n,1..dim];
  Unique                    : bvector[1..n];
  NumberOfOptis, i, ErrCode : integer;
begin
  writeln('Computing all global minimizers of the ', Name);
  write('Search interval      : '); read(SearchInterval);
  write('Tolerance (relative) : '); read(Tolerance);
  writeln;
  AllGOp(f, SearchInterval, Tolerance,
            Opti, Unique, NumberOfOptis, Minimum, ErrCode);
  for i:=1 to NumberOfOptis do
  begin
    write(Opti[i]);
    if unique[i] then
      writeln('encloses a locally unique candidate for a global minimizer!')
    else
      writeln('may contain a local or global minimizer!')
  end;
  writeln;
  if (NumberOfOptis <> 0) then
  begin
    writeln(Minimum);  writeln('encloses the global minimum value!');
  end;
  if ErrCode <> 0 then writeln(AllGOpErrMsg(ErrCode));
  writeln;  writeln(NumberOfOptis:1, ' interval enclosure(s)');  writeln;
  if (NumberOfOptis = 1) and (unique[1]) then
    writeln('We have validated that there is a unique global optimizer!');
end;

begin { Main program }
  compute(fBranin, 'Function of Branin', fDim);  writeln;  writeln;
  compute(fLevy,   'Function of Levy',   fDim);
end.
