{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
{----------------------------------------------------------------------------}
{ This is an implementation of Newton's Method for computing a stationary    }
{ point of a twice continuously differentiable multi-dimensional function.   }
{                                                                            }
{    given:      the function f(x) and the starting value x0                 }
{    iteration:  x[n+1] := x[n] - InvHf(x[n])*Gf(x[n]) , n = 0,1,...         }
{                                                                            }
{ where InvHf(x) denotes the inverse of the Hessian matrix of f(x).          }
{ All real operations are replaced by interval operations, all function and  }
{ derivative evaluations are calculated by differentiation arithmetic.       }
{----------------------------------------------------------------------------}

program hess_ex;

use
  i_ari,      { Interval arithmetic                }
  mv_ari,     { Real matrix/vector arithmetic      }
  mvi_ari,    { Interval matrix/vector arithmetic  }
  matinv,     { Matrix inversion                   }
  hess_ari;   { Hessian differentiation arithmetic }

const
  fDim = 2;
  nmax = 100;
var
  fx     : interval;
  x, Gfx : ivector[1..fDim];
  Hfx    : imatrix[1..fDim,1..fDim];
  InvHfx : rmatrix[1..fDim,1..fDim];
  n, Err : integer;

function f (x: HTvector) : HessType[0..fDim,0..fDim];
begin
  f:= 100*sqr(x[2] - sqr(x[1])) + sqr(x[1] - 1);
end;

begin
  writeln('Newton''s method for finding a stationary point of Rosenbrock''s');
  writeln('function:  f(x) = 100*sqr(x[2] - sqr(x[1])) + sqr(x[1] - 1)');
  writeln;
  write('Starting vector x = ');  read(x);
  writeln;
  writeln('Iteration:');
  fghEvalH(f, x, fx, Gfx, Hfx);
  n := 0;
  repeat
    n := n + 1;
    writeln('x: ', x, 'Gf(x): ', Gfx);
    MatInv(mid(Hfx), InvHfx, Err);
    if Err = 0 then
    begin
      x:= x - InvHfx * Gfx;
      fghEvalH(f, x, fx, Gfx, Hfx);
    end;
  until ((0 in Gfx[1]) and (0 in Gfx[2])) or (n >= nmax) or (Err <> 0);
  if Err = 0 then
    begin
      writeln;
      writeln('Stationary point: ', x, 'Gradient value: ', Gfx);
      writeln('Expected solution: ');
      writeln(' 1.0 '); writeln(' 1.0 ');
    end
  else
    writeln(MatInvErrMsg(Err));
end.
