{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
{----------------------------------------------------------------------------}
{ This is an implementation of Newton's Method for computing a zero of       }
{ a continuously differentiable multi-dimensional function.                  }
{                                                                            }
{    given:      the function f(x) and the starting value x[0]               }
{    iteration:  x[n+1] := x[n] - InvJf(x[n])*f(x[n]) , n = 0,1,...          }
{                                                                            }
{ where InvJf(x) denotes the inverse of the Jacobian matrix of f(x).         }
{ All real operations are replaced by interval operations, all function and  }
{ derivative evaluations are calculated by differentiation arithmetic.       }
{----------------------------------------------------------------------------}
program jac_ex;
use
  i_ari,       { Interval arithmetic                 }
  mv_ari,      { Real matrix/vector arithmetic       }
  mvi_ari,     { Interval matrix/vector arithmetic   }
  grad_ari,    { Gradient differentiation arithmetic }
  matinv;      { Matrix inversion                    }

const
  fDim = 2;
  nmax = 100;

var
  fx, x  : ivector[1..fDim];
  Jfx    : imatrix[1..fDim,1..fDim];
  InvJfx : rmatrix[1..fDim,1..fDim];
  n, Err : integer;

function f (x: GTvector) : GTvector[1..fDim,0..fDim];
var
  x1sqr : GradType[0..fDim];
begin
  x1sqr:= sqr(x[1]);
  f[1]:= ((6*x1sqr - intval(252)/10)*x1sqr + 24)*x[1] - 6*x[2];
  f[2]:= 12*x[2] - 6*x[1];
end;

begin
  writeln('Newton'' s method for finding a zero of Hansen''s function:');
  writeln('f1(x) = 6 x1^5 - 25.2 x1^3 + 24 x1 - 6 x2');
  writeln('f2(x) = 12 x2 - 6 x1');
  writeln;
  write('Starting vector x: ');  read (x);
  writeln('Iteration');
  fJEvalJ(f, x, fx, Jfx);
  n := 0;
  repeat
    n := n + 1;
    writeln('x: ', x, 'f(x): ', fx);
    MatInv(mid(Jfx), InvJfx, Err);
    if Err = 0 then
    begin
      x:= x - InvJfx * fx;
      fJEvalJ(f, x, fx, Jfx);
    end;
  until ((0 in fx[1]) and (0 in fx[2])) or (n >= nmax) or (Err <> 0);
  writeln;
  if Err = 0 then
    begin
     writeln('Zero: ', x, 'Function value: ', fx);
     writeln('Expected zeros:');
     writeln('-1.7475... or -1.0705... or 0.0 or 1.0705... or 1.7475...');
     writeln('-0.8737...    -0.5352...    0.0    0.5352... or 0.8737...');
    end
  else
    writeln(MatInvErrMsg(Err));
end.
