{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
{----------------------------------------------------------------------------}
{ This program uses module 'nlss' to compute the solutions of a system of    }
{ nonlinear equations given by Hansen. A starting interval vector and a      }
{ tolerance must be entered.                                                 }
{----------------------------------------------------------------------------}
program nlss_ex;

use
  i_ari,     { Interval arithmetic                 }
  mvi_ari,   { Interval matrix/vector arithmetic   }
  grad_ari,  { Gradient differentiation arithmetic }
  b_util,    { Boolean utilities                   }
  nlss;      { Nonlinear System Solver             }

const
  n    = 20;  { Maximum number of solutions to be computed }
  name = 'Hansen''s Function';

var
  fDim : integer;

{----------------------------------------------------------------------------}
{ Function to prompt for the desired value of 'fDim'. It is called only once }
{ within the declaration part of the first dynamic variable. This enables    }
{ full dynamics within the main program.                                     }
{----------------------------------------------------------------------------}
function set_fDim : integer;
begin
  write('Desired dimension    : ');  read(fDim);
  set_fDim := fDim;
end;

var
  SearchInterval   : ivector[1..set_fDim];        { Declaration using a call }
  Tolerance        : real;                        { of function 'set_fDim'.  }
  Solu             : imatrix[1..n,1..fDim];
  Unique           : bvector[1..n];
  NumberOfSolus, i : integer;
  ErrCode          : integer;

{----------------------------------------------------------------------------}
{ Definition of Hansen's function in 'fDim' variables.                       }
{----------------------------------------------------------------------------}
function f (x: GTvector) : GTvector[1..fDim,0..fDim];
var
  sqrsum : GradType[0..fDim];
  i      : integer;
begin
  sqrsum:= 0;
  for i:= 1 to fDim do
    sqrsum:= sqrsum + sqr(x[i]);
  for i:= 1 to fDim do
    f[i] := 6*x[i]/10 - 2 + 49*x[i]*sqrsum/100;
end;

{ Main program }
{--------------}
begin
  writeln;
  writeln('Computing all solutions for ', Name, ' in ', fDim, ' variables');
  writeln;
  write('Search interval      : ');  read(SearchInterval);
  write('Tolerance (relative) : ');  read(Tolerance);
  writeln;
  AllNLSS(f, SearchInterval, Tolerance, Solu, Unique, NumberOfSolus, ErrCode);
  for i:=1 to NumberOfSolus do
  begin
    write(Solu[i]);
    if unique[i] then
      writeln('encloses a locally unique zero!')
    else
      writeln('may contain a zero (not verified unique)!')
  end;
  if ErrCode <> 0 then writeln(AllNLSSErrMsg(ErrCode));
  writeln;  writeln(NumberOfSolus:1, ' interval enclosure(s)');  writeln;
  if (NumberOfSolus = 1) and (unique[1]) then
    writeln('We have validated that there is a globally unique zero!');
end.
