{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
{----------------------------------------------------------------------------}
{ Example for the evaluation of an arithmetic expression using procedure     }
{ 'Eval()'. The second order difference quotient for a real function f(x) is }
{ defined by Df(x,h) := (f(x-h) - 2f(x) + f(x+h)) / h^2. This quotient is    }
{ used to approximate the second derivative of                               }
{                                                                            }
{   f(x) := 540 (x^4 - 23x^3 + 159x^2 - 2x + 45) / (x^3 + 18x^2 + 501x + 20) }
{                                                                            }
{ at x = 1. Since f is twice differentiable, Df(1,h) should tend to          }
{ f''(1) = 36 if h tends to zero.                                            }
{----------------------------------------------------------------------------}
program ExprEval_Example_DiffQuot ( input, output );
use
  i_ari,        { Interval arithmetic   }
  expreval;     { Expression evaluation }

function f ( x : Staggered ) : Staggered;
begin
  f := 540 * (Power(x,4) - 23*Power(x,3) + 159*Power(x,2) - 2*x + 45) /
             (Power(x,3) + 18*Power(x,2) + 501*x + 20);
end;

function Df ( v : StaggArray ) : Staggered;
var
  x, h : Staggered;
begin
  x := v[1];
  h := v[2];
  Df := (f(x-h) - 2*f(x) + f(x+h)) / (h*h);
end;

var
  Eps, Approx        : real;
  StaggPrec, ErrCode : integer;
  Arg                : rvector[1..2];
  Encl               : interval;
begin
  writeln('Evaluation of the second order difference quotient ');
  writeln('   Df(x,h) = (f(x-h) - 2f(x) + f(x+h)) / h^2');
  writeln('for the function');
  write  ('   f(x) = 540(x^4 - 23x^3 + 159x^2 - 2x + 45) / ');
  writeln('(x^3 + 18x^2 + 501x + 20)');
  writeln('Note: f''''(1) = 36.');
  writeln;
  writeln('Enter the arguments:');
  write  ('   x = '); read(Arg[1]);
  write  ('   h = '); read(Arg[2]);
  writeln;
  writeln('Desired accuracy:');
  write  ('   Eps = '); read(Eps);
  writeln;

  Eval(Df,Arg,Eps,Approx,Encl,StaggPrec,ErrCode);
  if (ErrCode = 0) then
    begin
      writeln('Floating-point evaluation:  ', Approx);
      writeln('Interval enclosure:       ',   Encl);
      writeln('Corrections needed:          ',StaggPrec);
    end
  else
    writeln(EvalErrMsg(ErrCode));
end.
